"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deserializeCallData = deserializeCallData;
exports.makeSerializer = makeSerializer;
exports.serializeTransaction = exports.serializeCallData = void 0;

var _types = _interopRequireDefault(require("./serializer/src/types"));

var ops = _interopRequireWildcard(require("./serializer/src/operations"));

var _serializer = require("./serializer");

function isArrayType(type) {
  return type.indexOf('[]') !== -1;
}
/**
 * serialize call data
 * @param {String} action - call action
 * @param {Object} params - call params
 * @param {Object} abi
 */


var serializeCallData = function serializeCallData(action, params, abi) {
  var ser = makeSerializer(abi, action);
  return ser.toBuffer(params);
};
/**
 * deserialize call data
 * @param {String} action - call action
 * @param {String} data - data hex string
 * @param {Object} abi - contract abi
 */


exports.serializeCallData = serializeCallData;

function deserializeCallData(action, data, abi) {
  // construct serializer
  var serializer = makeSerializer(abi, action); // dserialize from hex

  return serializer.toObject(serializer.fromHex(data));
}

var serializeTransaction = function serializeTransaction(transaction) {
  return ops.transaction.toBuffer(ops.transaction.fromObject(transaction));
};
/**
 * make serializer by abi and action
 * @param {Object} abi
 * @param {String} action
 */


exports.serializeTransaction = serializeTransaction;

function makeSerializer(abi, action) {
  abi = JSON.parse(JSON.stringify(abi)); // let struct = Array.find(abi.structs, (s) => s.name === action);

  var struct = abi.structs.find(function (s) {
    return s.name === action;
  });
  var typeObj = {};
  struct.fields.forEach(function (f) {
    var isArrayFlag = false;

    if (isArrayType(f.type)) {
      isArrayFlag = true;
      f.type = f.type.split('[')[0];
    }

    var type = _types["default"][f.type];

    if (!type) {
      // let t = Array.find(abi.types, (t) => t.new_type_name === f.type);
      var t = abi.types.find(function (t) {
        return t.new_type_name === f.type;
      });

      if (t) {
        type = _types["default"][t.type];
      }
    }

    if (!type) {
      type = ops[f.type];
    }

    if (type) {
      if (isArrayFlag) {
        type = _types["default"].array(type, true);
      }
    }

    typeObj[f.name] = type;
  });
  return new _serializer.Serializer('temp', typeObj);
}